@@ -14,6 +14,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include "pari.h"
 #include "paripriv.h"
 
+#define DEBUGLEVEL DEBUGLEVEL_factor
+
 /* x,y two ZX, y non constant. Return q = x/y if y divides x in Z[X] and NULL
  * otherwise. If not NULL, B is a t_INT upper bound for ||q||_oo. */
 static GEN
@@ -354,12 +356,17 @@ END:
 static GEN
 shifteval(GEN Q, long n)
 {
+  pari_sp av = avma;
   long i, l = lg(Q);
   GEN s;
 
   if (!signe(Q)) return gen_0;
   s = gel(Q,l-1);
-  for (i = l-2; i > 1; i--) s = addii(gel(Q,i), shifti(s, n));
+  for (i = l-2; i > 1; i--)
+  {
+    s = addii(gel(Q,i), shifti(s, n));
+    if (gc_needed(av,1)) s = gerepileuptoint(av, s);
+  }
   return s;
 }
 
@@ -382,13 +389,14 @@ root_bound(GEN P0)
     set_avma(av);
   }
   if (k < 0) k = 0;
-  x = int2n(k);
   y = int2n(k+1);
+  if (d > 2000) return y; /* likely to be expensive, don't bother */
+  x = int2n(k);
   for(k=0; ; k++)
   {
     z = shifti(addii(x,y), -1);
     if (equalii(x,z) || k > 5) break;
-    if (cmpii(poleval(Q,z), mulii(lP, powiu(z, d))) < 0)
+    if (cmpii(ZX_Z_eval(Q,z), mulii(lP, powiu(z, d))) < 0)
       y = z;
     else
       x = z;
@@ -867,17 +875,19 @@ ZX_DDF(GEN x)
   return L;
 }
 
-/* SquareFree Factorization. f = prod P^e, all e distinct, in Z[X] (char 0
- * would be enough, if ZX_gcd --> ggcd). Return (P), set *ex = (e) */
+/* SquareFree Factorization in Z[X] (char 0 is enough, if ZX_gcd -> RgX_gcd)
+ * f = prod Q[i]^E[i], E[1] < E[2] < ..., and Q[i] squarefree and coprime.
+ * Return Q, set *pE = E. For efficiency, caller should have used ZX_valrem
+ * so that f(0) != 0 */
 GEN
-ZX_squff(GEN f, GEN *ex)
+ZX_squff(GEN f, GEN *pE)
 {
-  GEN T, V, P, e;
-  long i, k, val = ZX_valrem(f, &f), n = 2 + degpol(f);
+  GEN T, V, P, E;
+  long i, k, n = 1 + degpol(f);
 
   if (signe(leading_coeff(f)) < 0) f = ZX_neg(f);
-  e = cgetg(n,t_VECSMALL);
-  P = cgetg(n,t_COL);
+  E = cgetg(n, t_VECSMALL);
+  P = cgetg(n, t_COL);
   T = ZX_gcd_all(f, ZX_deriv(f), &V);
   for (k = i = 1;; k++)
   {
@@ -889,7 +899,7 @@ ZX_squff(GEN f, GEN *ex)
      * T = prod_{i: e_i > k} T_i^{e_i - k} */
     if (!dW)
     {
-      if (dV) { gel(P,i) = Q_primpart(V); e[i] = k; i++; }
+      if (dV) { gel(P,i) = Q_primpart(V); E[i] = k; i++; }
       break;
     }
     if (dW == dV)
@@ -900,51 +910,55 @@ ZX_squff(GEN f, GEN *ex)
     else
     {
       gel(P,i) = Q_primpart(RgX_div(V,W));
-      e[i] = k; i++; V = W;
+      E[i] = k; i++; V = W;
     }
   }
-  if (val) { gel(P,i) = pol_x(varn(f)); e[i] = val; i++;}
   setlg(P,i);
-  setlg(e,i); *ex = e; return P;
+  setlg(E,i); *pE = E; return P;
 }
 
 static GEN
-fact_from_DDF(GEN fa, GEN e, long n)
+fact_from_DDF(GEN Q, GEN E, long n)
 {
   GEN v,w, y = cgetg(3, t_MAT);
-  long i,j,k, l = lg(fa);
+  long i,j,k, l = lg(Q);
 
-  v = cgetg(n+1,t_COL); gel(y,1) = v;
-  w = cgetg(n+1,t_COL); gel(y,2) = w;
-  for (k=i=1; i<l; i++)
+  v = cgetg(n+1, t_COL); gel(y,1) = v;
+  w = cgetg(n+1, t_COL); gel(y,2) = w;
+  for (k = i = 1; i < l; i++)
   {
-    GEN L = gel(fa,i), ex = utoipos(e[i]);
+    GEN L = gel(Q,i), e = utoipos(E[i]);
     long J = lg(L);
-    for (j=1; j<J; j++,k++)
+    for (j = 1; j < J; j++,k++)
     {
-      gel(v,k) = gcopy(gel(L,j));
-      gel(w,k) = ex;
+      gel(v,k) = ZX_copy(gel(L,j));
+      gel(w,k) = e;
     }
   }
   return y;
 }
 
-/* Factor x in Z[t] */
+/* Factor T in Z[x] */
 static GEN
-ZX_factor_i(GEN x)
+ZX_factor_i(GEN T)
 {
-  GEN fa,ex,y;
-  long n,i,l;
+  GEN Q, E, y;
+  long n, i, l, v;
 
-  if (!signe(x)) return prime_fact(x);
-  fa = ZX_squff(x, &ex);
-  l = lg(fa); n = 0;
-  for (i=1; i<l; i++)
+  if (!signe(T)) return prime_fact(T);
+  v = ZX_valrem(T, &T);
+  Q = ZX_squff(T, &E); l = lg(Q);
+  for (i = 1, n = 0; i < l; i++)
   {
-    gel(fa,i) = ZX_DDF(gel(fa,i));
-    n += lg(gel(fa,i))-1;
+    gel(Q,i) = ZX_DDF(gel(Q,i));
+    n += lg(gel(Q,i)) - 1;
   }
-  y = fact_from_DDF(fa,ex,n);
+  if (v)
+  {
+    Q = vec_append(Q, mkvec(pol_x(varn(T))));
+    E = vecsmall_append(E, v); n++;
+  }
+  y = fact_from_DDF(Q, E, n);
   return sort_factor_pol(y, cmpii);
 }
 GEN
@@ -987,7 +1001,7 @@ nfrootsQ(GEN x)
   RgX_check_ZX(x,"nfrootsQ");
   val = ZX_valrem(x, &x);
   z = DDF_roots( ZX_radical(x) );
-  if (val) z = shallowconcat(z, gen_0);
+  if (val) z = vec_append(z, gen_0);
   return gerepileupto(av, sort(z));
 }
 
@@ -1082,9 +1096,7 @@ ZX_gcd_slice(GEN A, GEN B, GEN g, GEN P, GEN *mod)
   if (ZX_gcd_filter(&H, &P))
     T = ZV_producttree(P);
   H = nxV_chinese_center_tree(H, P, T, ZV_chinesetree(P, T));
-  *mod = gmael(T, lg(T)-1, 1);
-  gerepileall(av, 2, &H, mod);
-  return H;
+  *mod = gmael(T, lg(T)-1, 1); return gc_all(av, 2, &H, mod);
 }
 
 GEN
@@ -1334,99 +1346,74 @@ polcyclofactors(GEN f)
   set_avma(av); return cgetg(1,t_VEC);
 }
 
-/* return t*x mod T(x), T a monic ZX. Assume deg(t) < deg(T) */
+/* list of all squarefree odd x such that phi(x) = n, P^-(x) > m. Unsorted */
 static GEN
-ZXQ_mul_by_X(GEN t, GEN T)
+invphi(ulong n, ulong m)
 {
-  GEN lt;
-  t = RgX_shift_shallow(t, 1);
-  if (degpol(t) < degpol(T)) return t;
-  lt = leading_coeff(t);
-  if (is_pm1(lt)) return signe(lt) > 0 ? ZX_sub(t, T): ZX_add(t, T);
-  return ZX_sub(t, ZX_Z_mul(T, leading_coeff(t)));
-}
-/* f a product of Phi_n, all n odd; deg f > 1. Is it irreducible ? */
-static long
-BD_odd_iscyclo(GEN f)
-{
-  pari_sp av;
-  long d, e, n, bound;
-  GEN t;
-  f = ZX_deflate_max(f, &e);
-  av = avma;
-  /* The original f is cyclotomic (= Phi_{ne}) iff the present one is Phi_n,
-   * where all prime dividing e also divide n. If current f is Phi_n,
-   * then n is odd and squarefree */
-  d = degpol(f); /* = phi(n) */
-  /* Let e > 0, g multiplicative such that
-       g(p) = p / (p-1)^(1+e) < 1 iff p < (p-1)^(1+e)
-     For all squarefree odd n, we have g(n) < C, hence n < C phi(n)^(1+e), where
-       C = \prod_{p odd | p > (p-1)^(1+e)} g(p)
-     For e = 1/10,   we obtain p = 3, 5 and C < 1.523
-     For e = 1/100,  we obtain p = 3, 5, ..., 29 and C < 2.573
-     In fact, for n <= 10^7 odd & squarefree, we have n < 2.92 * phi(n)
-     By the above, n<10^7 covers all d <= (10^7/2.573)^(1/(1+1/100)) < 3344391.
-  */
-  if (d <= 3344391)
-    bound = (long)(2.92 * d);
-  else
-    bound = (long)(2.573 * pow(d,1.01));
-  /* IF f = Phi_n, n squarefree odd, then n <= bound */
-  t = pol_xn(d-1, varn(f));
-  for (n = d; n <= bound; n++)
+  GEN C, D;
+  long l, i;
+  if (n == 1) return mkvecsmall(1);
+  D = divisorsu(n); l = lg(D);
+  C = cgetg(1, t_VECSMALL);
+  for (i = 2; i < l; i++) /* skip 1 */
   {
-    t = ZXQ_mul_by_X(t, f);
-    /* t = (X mod f(X))^d */
-    if (degpol(t) == 0) break;
-    if (gc_needed(av,1))
-    {
-      if(DEBUGMEM>1) pari_warn(warnmem,"BD_odd_iscyclo");
-      t = gerepilecopy(av, t);
-    }
+    ulong d = D[i], p;
+    if (d < m) continue;
+    p = d + 1; if (!uisprime(p)) continue;
+    C = vecsmall_concat(C, zv_z_mul(invphi(D[l-i], p), p));
   }
-  if (n > bound || eulerphiu(n) != (ulong)d) return 0;
-
-  if (e > 1) return (u_ppo(e, n) == 1)? e * n : 0;
-  return n;
+  return C;
 }
 
-/* Checks if f, monic squarefree ZX with |constant coeff| = 1, is a cyclotomic
- * polynomial. Returns n if f = Phi_n, and 0 otherwise */
-static long
-BD_iscyclo(GEN f)
-{
-  pari_sp av = avma;
-  GEN f2, fn, f1;
-
-  if (degpol(f) == 1) return isint1(gel(f,2))? 2: 1;
-  f1 = ZX_graeffe(f);
-  /* f = product of Phi_n, n odd */
-  if (ZX_equal(f, f1)) return gc_long(av, BD_odd_iscyclo(f));
-
-  fn = ZX_z_unscale(f, -1); /* f(-x) */
-  /* f = product of Phi_n, n = 2 mod 4 */
-  if (ZX_equal(f1, fn)) return gc_long(av, 2*BD_odd_iscyclo(fn));
-
-  if (issquareall(f1, &f2))
-  {
-    GEN lt = leading_coeff(f2);
-    long c;
-    if (signe(lt) < 0) f2 = ZX_neg(f2);
-    c = BD_iscyclo(f2);
-    return odd(c)? 0: 2*c;
-  }
-  return gc_long(av, 0);
-}
 long
 poliscyclo(GEN f)
 {
-  long d;
+  const ulong p = 2147483647; /* prime */
+  pari_sp av;
+  long i, n, e, l;
+  ulong f3, fm3;
+  GEN D, fp, _3;
   if (typ(f) != t_POL) pari_err_TYPE("poliscyclo", f);
-  d = degpol(f);
-  if (d <= 0 || !RgX_is_ZX(f)) return 0;
-  if (!equali1(gel(f,d+2)) || !is_pm1(gel(f,2))) return 0;
-  if (d == 1) return signe(gel(f,2)) > 0? 2: 1;
-  return ZX_is_squarefree(f)? BD_iscyclo(f): 0;
+  n = degpol(f);
+  if (n <= 0 || !RgX_is_ZX(f)) return 0;
+  if (!equali1(gel(f,n+2)) || !is_pm1(gel(f,2))) return 0;
+  if (n == 1) return signe(gel(f,2)) > 0? 2: 1;
+  av = avma;
+  f = ZX_deflate_max(f, &e); if (e != 1) n = degpol(f);
+  D = invphi(n, 1); /* squareefree odd d s.t. phi(d) = n */
+  l = lg(D); _3 = gmodulss(3, p);
+  fp = ZX_to_Flx(f, p);
+  f3 = Flx_eval(fp, 3, p);
+  fm3 = Flx_eval(fp, p-3, p);
+  /* f(x^e) is cyclotomic (= Phi_{de}) iff f = Phi_d, where all prime dividing
+   * e also divide d. */
+  for (i = 1; i < l; i++)
+  {
+    long d = D[i]; /* squarefree odd */
+    if (odd(e))
+    {
+      if (e == 1 || u_ppo(e, d) == 1)
+      { /* early abort: check whether f(3) = Phi_d(3) or Phi_2d(3) = Phi_d(-3)
+         * mod p before checking in Z. N.B. phi(d) and value at 3 mod p
+         * determine Phi_d for all d <= 10^7 */
+        ulong F3 = Rg_to_Fl(polcyclo_eval(d, _3), p);
+        if (F3 == f3 && ZX_equal(f, polcyclo(d, varn(f))))
+          return gc_long(av, d * e);
+        if (F3 == fm3 && ZX_equal(f, polcyclo(2*d, varn(f))))
+          return gc_long(av, 2* d * e);
+      }
+    }
+    else
+    {
+      if (u_ppo(e, 2*d) == 1)
+      { /* early abort: check whether f(3) = Phi_2d(3) mod p */
+        ulong F3 = Rg_to_Fl(polcyclo_eval(2*d, _3), p);
+        if (F3 == f3 && ZX_equal(f, polcyclo(2*d, varn(f))))
+          return gc_long(av, 2* d * e);
+      }
+    }
+  }
+  return gc_long(av, 0);
 }
 
 long
