@@ -852,7 +852,8 @@ find_isogenous_from_Atkin(GEN a4, GEN a6, ulong ell, struct meqn *MEQN, GEN g, G
       GEN a4t, a6t, h;
       a4a6t(&a4t, &a6t, ell, E4t, E6t, T, p);
       h = find_kernel(a4, a6, ell, a4t, a6t, pp1, T, p, pp, e);
-      if (h) return gerepilecopy(ltop, mkvec3(a4t, a6t, h));
+      if (h && signe(Fq_elldivpolmod(a4, a6, ell, h, T, pp))==0)
+        return gerepilecopy(ltop, mkvec3(a4t, a6t, h));
     }
   }
   pari_err_BUG("find_isogenous_from_Atkin, kernel not found");
