@@ -2266,6 +2266,10 @@ static ulong
 ZXQX_resultant_bound(GEN nf, GEN A, GEN B)
 { return ZXQX_resultant_bound_i(nf, A, B, &RgX_RgXY_ResBound); }
 
+static GEN
+_ZXQ_powu(GEN x, ulong u, GEN T)
+{ return typ(x) == t_INT? powiu(x, u): ZXQ_powu(x, u, T); }
+
 /* Compute Res(A, B/dB) in Z[X]/T, assuming A,B in Z[X,Y], dB in Z or NULL (= 1)
  * If B=NULL, take B = A' and assume deg A > 1 */
 static GEN
@@ -2278,8 +2282,8 @@ ZXQX_resultant_all(GEN A, GEN B, GEN T, GEN dB, ulong bound)
   {
     long a = degpol(A), b = degpol(B);
     if (a < 0 || b < 0) return gen_0;
-    if (!a) return gpowgs(gel(A,2), b);
-    if (!b) return gpowgs(gel(B,2), a);
+    if (!a) return _ZXQ_powu(gel(A,2), b, T);
+    if (!b) return _ZXQ_powu(gel(B,2), a, T);
   } else
     if (!bound) B = RgX_deriv(A);
   if (!bound) bound = ZXQX_resultant_bound(nfinit(T, DEFAULTPREC), A, B);
@@ -2300,8 +2304,8 @@ nfX_resultant(GEN nf, GEN x, GEN y)
   pari_sp av = avma;
   GEN cx, cy, D, T = nf_get_pol(nf);
   ulong bound;
-  long d = degpol(x), v = varn(T);
-  if (d <= 1) return d == 1? pol_1(v): pol_0(v);
+  long dx = degpol(x), dy = degpol(y), v = varn(T);
+  if (dx < 0 || dy < 0) return pol_0(v);
   x = Q_primitive_part(x, &cx);
   y = Q_primitive_part(y, &cy);
   bound = ZXQX_resultant_bound(nf, x, y);
