@@ -21,6 +21,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include "pari.h"
 #include "paripriv.h"
 
+#define DEBUGLEVEL DEBUGLEVEL_qfsolve
+
 /* LINEAR ALGEBRA */
 /* complete by 0s, assume l-1 <= n */
 static GEN
@@ -54,10 +56,7 @@ completebasis(GEN Mv, long redflag)
                        vecslice(U, m-n+1,m));
 }
 
-/* Compute the kernel of M mod p.
- * returns [d,U], where
- * d = dim (ker M mod p)
- * U in GLn(Z), and its first d columns span the kernel. */
+/* Return U in GLn(Z) whose first d columns span Ker (M mod p). */
 static GEN
 kermodp(GEN M, GEN p, long *d)
 {
@@ -85,111 +84,41 @@ det_minors(GEN G)
   return v;
 }
 
-/* Given a symmetric matrix G over Z, compute the Witt invariant
- *  of G at the prime p (at real place if p = NULL)
- * Assume that none of the determinant G[1..i,1..i] is 0. */
-static long
-qflocalinvariant(GEN G, GEN p)
-{
-  long i, j, c, l = lg(G);
-  GEN diag, v = det_minors(G);
-  /* Diagonalize G first. */
-  diag = cgetg(l, t_VEC);
-  for (i = 1; i < l; i++) gel(diag,i) = mulii(gel(v,i+1), gel(v,i));
-
-  /* Then compute the product of the Hilbert symbols */
-  /* (diag[i],diag[j])_p for i < j */
-  c = 1;
-  for (i = 1; i < l-1; i++)
-    for (j = i+1; j < l; j++)
-      if (hilbertii(gel(diag,i), gel(diag,j), p) < 0) c = -c;
-  return c;
-}
-
 static GEN
-hilberts(GEN a, GEN b, GEN P, long lP)
+hilberts(GEN a, GEN b, GEN P)
 {
+  long i, lP = lg(P);
   GEN v = cgetg(lP, t_VECSMALL);
-  long i;
   for (i = 1; i < lP; i++) v[i] = hilbertii(a, b, gel(P,i)) < 0;
   return v;
 }
 
-/* G symmetrix matrix or qfb or list of quadratic forms with same discriminant.
- * P must be equal to factor(-abs(2*matdet(G)))[,1]. */
+/* 4 | disc(q); special case of gtomat */
 static GEN
-qflocalinvariants(GEN G, GEN P)
+qfbmat(GEN q)
 {
-  GEN sol;
-  long i, j, l, lP = lg(P);
-
-  /* convert G into a vector of symmetric matrices */
-  G = (typ(G) == t_VEC)? shallowcopy(G): mkvec(G);
-  l = lg(G);
-  for (j = 1; j < l; j++)
-  {
-    GEN g = gel(G,j);
-    if (is_qfb_t(typ(g))) gel(G,j) = gtomat(g);
-  }
-  sol = cgetg(l, t_MAT);
-  if (lg(gel(G,1)) == 3)
-  { /* in dimension 2, each invariant is a single Hilbert symbol. */
-    GEN d = negi(ZM_det(gel(G,1)));
-    for (j = 1; j < l; j++)
-    {
-      GEN a = gcoeff(gel(G,j),1,1);
-      gel(sol,j) = hilberts(a, d, P, lP);
-    }
-  }
-  else /* in dimension n > 2, we compute a product of n Hilbert symbols. */
-    for (j = 1; j <l; j++)
-    {
-      GEN g = gel(G,j), v = det_minors(g), w = cgetg(lP, t_VECSMALL);
-      long n = lg(v);
-      gel(sol,j) = w;
-      for (i = 1; i < lP; i++)
-      {
-        GEN p = gel(P,i);
-        long k = n-2, h = hilbertii(gel(v,k), gel(v,k+1),p);
-        for (k--; k >= 1; k--)
-          if (hilbertii(negi(gel(v,k)), gel(v,k+1),p) < 0) h = -h;
-        w[i] = h < 0;
-      }
-    }
-  return sol;
+  GEN a = gel(q,1), b = shifti(gel(q,2), -1), c = gel(q,3);
+  return mkmat2(mkcol2(a, b), mkcol2(b, c));
+}
+/* 2*qfbmat(q) */
+static GEN
+qfbmat2(GEN q)
+{
+  GEN a = shifti(gel(q,1), 1), b = gel(q,2), c = shifti(gel(q,3), 1);
+  return mkmat2(mkcol2(a, b), mkcol2(b, c));
+}
+/* Given a symmetric matrix G over Z, compute the Witt invariant of G at p
+ * v = det_minors(G) [G diagonalized]; assume that none of the v[i] is 0. */
+static long
+witt(GEN v, GEN p)
+{
+  long k = lg(v)-2, h = hilbertii(gel(v,k), gel(v,k+1), p);
+  for (k--; k >= 1; k--)
+    if (hilbertii(negi(gel(v,k)), gel(v,k+1),p) < 0) h = -h;
+  return h;
 }
 
 /* QUADRATIC FORM REDUCTION */
-static GEN
-qfb(GEN D, GEN a, GEN b, GEN c)
-{
-  if (signe(D) < 0) return mkqfi(a,b,c);
-  retmkqfr(a,b,c,real_0(DEFAULTPREC));
-}
-
-/* Gauss reduction of the binary quadratic form
- * Q = a*X^2+2*b*X*Y+c*Y^2 of discriminant D (divisible by 4)
- * returns the reduced form */
-static GEN
-qfbreduce(GEN D, GEN Q)
-{
-  GEN a = gel(Q,1), b = shifti(gel(Q,2),-1), c = gel(Q,3);
-  while (signe(a))
-  {
-    GEN r, q, nexta, nextc;
-    q = dvmdii(b,a, &r); /* FIXME: export as dvmdiiround ? */
-    if (signe(r) > 0 && abscmpii(shifti(r,1), a) > 0)
-    {
-      r = subii(r, absi_shallow(a));
-      q = addis(q, signe(a));
-    }
-    nextc = a; nexta = subii(c, mulii(q, addii(r,b)));
-    if (abscmpii(nexta, a) >= 0) break;
-    c = nextc; b = negi(r); a = nexta;
-  }
-  return qfb(D,a,shifti(b,1),c);
-}
-
 /* private version of qfgaussred:
  * - early abort if k-th principal minor is singular, return stoi(k)
  * - else return a matrix whose upper triangular part is qfgaussred(a) */
@@ -301,7 +230,7 @@ qflllgram_indefgoon(GEN G)
   V = rowpermute(vecslice(G3, 2,n-1), mkvecsmall2(1,n));
   A = mkmat2(mkcol2(gcoeff(G3,1,1),gcoeff(G3,1,n)),
              mkcol2(gcoeff(G3,1,n),gcoeff(G3,2,2)));
-  B = ground(RgM_neg(RgM_mul(RgM_inv(A), V)));
+  B = ground(RgM_neg(QM_mul(QM_inv(A), V)));
   U3 = matid(n);
   for (j = 2; j < n; j++)
   {
@@ -400,15 +329,33 @@ ZsymM_Z_divexact(GEN G, GEN d)
 {
   long i,j,l = lg(G);
   GEN H = cgetg(l, t_MAT);
-  for(j=1; j<l; j++)
+  for (j = 1; j < l; j++)
   {
     GEN c = cgetg(l, t_COL), b = gel(G,j);
-    for(i=1; i<j; i++) gcoeff(H,j,i) = gel(c,i) = diviiexact(gel(b,i),d);
+    for (i=1; i < j; i++) gcoeff(H,j,i) = gel(c,i) = diviiexact(gel(b,i),d);
     gel(c,j) = diviiexact(gel(b,j),d);
     gel(H,j) = c;
   }
   return H;
 }
+/* by blocks, in place: G[1,1] /= d, G[2,2] *= d */
+static void
+ZsymM_Z_divexact_partial(GEN G, long n,  GEN d)
+{
+  long i, j, l = lg(G);
+  for(j = 1; j <= n; j++)
+  {
+    for (i = 1; i < j; i++)
+      gcoeff(G,i,j) = gcoeff(G,j,i) = diviiexact(gcoeff(G,i,j), d);
+    gcoeff(G,j,j) = diviiexact(gcoeff(G,j,j), d);
+  }
+  for (; j < l; j++)
+  {
+    for (i = n+1; i < j; i++)
+      gcoeff(G,i,j) = gcoeff(G,j,i) = mulii(gcoeff(G,i,j), d);
+    gcoeff(G,j,j) = mulii(gcoeff(G,j,j), d);
+  }
+}
 
 /* write symmetric G as [A,B;B~,C], A dxd, C (n-d)x(n-d) */
 static void
@@ -419,6 +366,36 @@ blocks4(GEN G, long d, long n, GEN *A, GEN *B, GEN *C)
   *B = rowslice(G2, 1, d);
   *C = rowslice(G2, d+1, n);
 }
+static GEN qfsolvemodp(GEN G, GEN p);
+static void
+update_fm(GEN f, GEN a, long i)
+{
+  if (!odd(i))
+    gel(f,1) = a;
+  else
+  {
+    long v = vals(i+1), k;
+    GEN b = gel(f,1), u = QM_mul(b, a);
+    gel(f,1) = gen_0;
+    if (v+2 >= lg(f)) pari_err_BUG("update_fm");
+    for (k = 1; k < v; k++)
+    {
+      u = QM_mul(gel(f, k+1), u);
+      gel(f,k+1) = gen_0; /* for gerepileall */
+    }
+    gel(f,v+1) = u;
+  }
+}
+static GEN
+prod_fm(GEN f, long i)
+{
+  long v = vals(i), k;
+  GEN u = gel(f, ++v);
+  for (i >>= v, k = v+1; i; i >>= 1, k++)
+    if (odd(i)) u = QM_mul(gel(f,k), u);
+  return u;
+}
+
 /* Minimization of the quadratic form G, deg G != 0, dim n >= 2
  * G symmetric integral
  * Returns [G',U,factd] with U in GLn(Q) such that G'=U~*G*U*constant
@@ -426,221 +403,249 @@ blocks4(GEN G, long d, long n, GEN *A, GEN *B, GEN *C)
  * In dimension 3 or 4, may return a prime p if the reduction at p is
  * impossible because of local nonsolvability.
  * P,E = factor(+/- det(G)), "prime" -1 is ignored. Destroy E. */
-static GEN qfsolvemodp(GEN G, GEN p);
 static GEN
-qfminimize(GEN G, GEN P, GEN E)
+qfminimize_fact(GEN G, GEN P, GEN E, long loc)
 {
-  pari_sp av;
-  GEN d, U, Ker, sol, aux, faE, faP;
-  long n = lg(G)-1, lP = lg(P), i, dimKer, m;
+  GEN U = NULL, Ker = NULL, faE, faP;
+  long n = lg(G)-1, lP = lg(P), i;
   faP = vectrunc_init(lP);
   faE = vecsmalltrunc_init(lP);
-  av = avma;
-  U = NULL;
   for (i = 1; i < lP; i++)
   {
     GEN p = gel(P,i);
-    long vp = E[i];
-    if (!vp || !p) continue;
-
-    if (DEBUGLEVEL >= 4) err_printf("    p^v = %Ps^%ld\n", p,vp);
-    /* The case vp = 1 can be minimized only if n is odd. */
-    if (vp == 1 && n%2 == 0) {
-      vectrunc_append(faP, p);
-      vecsmalltrunc_append(faE, 1);
-      av = avma; continue;
-    }
-    Ker = kermodp(G,p, &dimKer); /* dimKer <= vp */
-    if (DEBUGLEVEL >= 4) err_printf("    dimKer = %ld\n",dimKer);
-    if (dimKer == n)
-    { /* trivial case: dimKer = n */
-      if (DEBUGLEVEL >= 4) err_printf("     case 0: dimKer = n\n");
-      G = ZsymM_Z_divexact(G, p);
-      E[i] -= n;
-      i--; continue; /* same p */
-    }
-    G = qf_apply_ZM(G, Ker);
-    U = U? RgM_mul(U,Ker): Ker;
-
-    /* 1st case: dimKer < vp */
-    /* then the kernel mod p contains a kernel mod p^2 */
-    if (dimKer < vp)
+    long vp = E[i], wp = vp;
+    if (!vp) continue;
+    if (DEBUGLEVEL >= 3) err_printf("qfminimize: for %Ps^%ld:", p,vp);
+    while (vp)
     {
-      if (DEBUGLEVEL >= 4) err_printf("    case 1: dimker < vp\n");
-      if (dimKer == 1)
+      long idx = 0, dimKer = 0; /* -Wall */
+      GEN d, sol = NULL, FU = zerovec(2*expu(vp)+100);
+      pari_sp av = avma;
+      while (vp) /* loop until vp <= n */
+      {
+        if (DEBUGLEVEL>=3 && vp <= wp)
+        { err_printf(" %ld%%", (E[i]-vp)*100/E[i]); wp -= E[i]/100; }
+        /* The case vp = 1 can be minimized only if n is odd. */
+        if (vp == 1 && !odd(n)) break;
+        Ker = kermodp(G,p, &dimKer); /* dimKer <= vp */
+        if (DEBUGLEVEL >= 4) err_printf("    dimKer = %ld\n",dimKer);
+        if (dimKer == n) break;
+        G = qf_apply_ZM(G, Ker);
+        /* 1st case: dimKer < vp */
+        /* then the kernel mod p contains a kernel mod p^2 */
+        if (dimKer >= vp) break;
+
+        if (DEBUGLEVEL >= 4) err_printf("    case 1: dimker < vp\n");
+        if (dimKer == 1)
+        {
+          long j;
+          gcoeff(G,1,1) = diviiexact(gcoeff(G,1,1), sqri(p));
+          for (j = 2; j <= n; j++)
+            gcoeff(G,1,j) = gcoeff(G,j,1) = diviiexact(gcoeff(G,j,1), p);
+          gel(Ker,1) = RgC_Rg_div(gel(Ker,1), p);
+          vp -= 2;
+        }
+        else
+        {
+          GEN A, B, C, K2;
+          long j, dimKer2;
+          blocks4(G, dimKer,n, &A,&B,&C); A = ZsymM_Z_divexact(A, p);
+          K2 = kermodp(A, p, &dimKer2);
+          /* Write G = [pA,B;B~,C] and apply [K2/p,0;0,Id] by blocks */
+          A = qf_apply_ZM(A,K2); ZsymM_Z_divexact_partial(A, dimKer2, p);
+          B = ZM_transmul(B,K2);
+          for (j = 1; j <= dimKer2; j++) gel(B,j) = ZC_Z_divexact(gel(B,j), p);
+          G = shallowmatconcat(mkmat2(mkcol2(A,B), mkcol2(shallowtrans(B), C)));
+          /* Ker *= [K2,0;0,Id] */
+          B = ZM_mul(vecslice(Ker,1,dimKer),K2);
+          for (j = 1; j <= dimKer2; j++) gel(B,j) = RgC_Rg_div(gel(B,j), p);
+          Ker = shallowconcat(B, vecslice(Ker,dimKer+1,n));
+          vp -= 2*dimKer2;
+        }
+        update_fm(FU, Ker, idx++);
+        if (gc_needed(av, 1))
+        {
+          if (DEBUGMEM >= 2) pari_warn(warnmem,"qfminimize");
+          gerepileall(av, 2, &G, &FU);
+        }
+      }
+      if (idx)
+      {
+        GEN PU = prod_fm(FU, idx);
+        U = U ? QM_mul(U, PU): PU;
+      }
+      if (vp == 0) break;
+      if (vp == 1 && !odd(n))
+      {
+        vectrunc_append(faP, p);
+        vecsmalltrunc_append(faE, 1);
+        break;
+      }
+      if (dimKer == n)
+      { /* trivial case: dimKer = n */
+        if (DEBUGLEVEL >= 4) err_printf("     case 0: dimKer = n\n");
+        G = ZsymM_Z_divexact(G, p);
+        vp -= n; continue;
+      }
+      U = U ? QM_mul(U, Ker): Ker;
+      /* vp = dimKer
+       * 2nd case: kernel has dim >= 2 and contains an elt of norm 0 mod p^2,
+       * find it */
+      if (dimKer > 2) {
+        if (DEBUGLEVEL >= 4) err_printf("    case 2.1\n");
+        dimKer = 3;
+        sol = qfsolvemodp(ZsymM_Z_divexact(principal_minor(G,3),p),  p);
+        sol = FpC_red(sol, p);
+      }
+      else if (dimKer == 2)
+      {
+        GEN a = modii(diviiexact(gcoeff(G,1,1),p), p);
+        GEN b = modii(diviiexact(gcoeff(G,1,2),p), p);
+        GEN c = modii(diviiexact(gcoeff(G,2,2),p), p);
+        GEN D = modii(subii(sqri(b), mulii(a,c)), p);
+        if (kronecker(D,p) >= 0)
+        {
+          if (DEBUGLEVEL >= 4) err_printf("    case 2.2\n");
+          sol = signe(a)? mkcol2(Fp_sub(Fp_sqrt(D,p), b, p), a): vec_ei(2,1);
+        }
+      }
+      if (sol)
       {
         long j;
-        gel(G,1) = ZC_Z_divexact(gel(G,1), p);
-        for (j = 1; j<=n; j++) gcoeff(G,1,j) = diviiexact(gcoeff(G,1,j), p);
-        gel(U,1) = RgC_Rg_div(gel(U,1), p);
-        E[i] -= 2;
+        sol = FpC_center(sol, p, shifti(p,-1));
+        sol = Q_primpart(sol);
+        if (DEBUGLEVEL >= 4) err_printf("    sol = %Ps\n", sol);
+        Ker = completebasis(vecextend(sol,n), 1);
+        G = qf_apply_ZM(G, Ker);
+        for (j = 1; j < n; j++)
+          gcoeff(G,n,j) = gcoeff(G,j,n) = diviiexact(gcoeff(G,j,n), p);
+        gcoeff(G,n,n) = diviiexact(gcoeff(G,n,n), sqri(p));
+        U = QM_mul(U,Ker); gel(U,n) = RgC_Rg_div(gel(U,n), p);
+        vp -= 2; continue;
       }
-      else
+      /* Now 0 < vp = dimKer < 3 and kernel contains no vector with norm p^2 */
+      /* exchanging kernel and image makes minimization easier ? */
+      d = ZM_det(G); if (odd((n-3) / 2)) d = negi(d);
+      if ((vp==1 && kronecker(gmod(gdiv(negi(d), gcoeff(G,1,1)),p), p) >= 0)
+          || (vp==2 && odd(n) && n >= 5)
+          || (vp==2 && !odd(n) && kronecker(modii(diviiexact(d,sqri(p)), p),p) < 0))
       {
-        GEN A,B,C, K2 = ZsymM_Z_divexact(principal_minor(G,dimKer),p);
-        long j, dimKer2;
-        K2 = kermodp(K2, p, &dimKer2);
-        for (j = dimKer2+1; j <= dimKer; j++) gel(K2,j) = ZC_Z_mul(gel(K2,j),p);
-        /* Write G = [A,B;B~,C] and apply [K2,0;0,p*Id]/p by blocks */
-        blocks4(G, dimKer,n, &A,&B,&C);
-        A = ZsymM_Z_divexact(qf_apply_ZM(A,K2), sqri(p));
-        B = ZM_Z_divexact(ZM_transmul(B,K2), p);
-        G = shallowmatconcat(mkmat2(mkcol2(A,B),
-                                    mkcol2(shallowtrans(B), C)));
-        /* U *= [K2,0;0,Id] */
-        U = shallowconcat(RgM_Rg_div(RgM_mul(vecslice(U,1,dimKer),K2), p),
-                          vecslice(U,dimKer+1,n));
-        E[i] -= 2*dimKer2;
+        long j;
+        if (DEBUGLEVEL >= 4) err_printf("    case 3\n");
+        ZsymM_Z_divexact_partial(G, dimKer, p);
+        for (j = dimKer+1; j <= n; j++) gel(U,j) = RgC_Rg_mul(gel(U,j), p);
+        vp -= 2*dimKer-n; continue;
       }
-      gerepileall(av, 2, &G, &U);
-      i--; continue; /* same p */
-    }
 
-   /* vp = dimKer
-    * 2nd case: kernel has dim >= 2 and contains an element of norm 0 mod p^2
-    * search for an element of norm p^2... in the kernel */
-    sol = NULL;
-    if (dimKer > 2) {
-      if (DEBUGLEVEL >= 4) err_printf("    case 2.1\n");
-      dimKer = 3;
-      sol = qfsolvemodp(ZsymM_Z_divexact(principal_minor(G,3),p),  p);
-      sol = FpC_red(sol, p);
-    }
-    else if (dimKer == 2)
-    {
-      GEN a = modii(diviiexact(gcoeff(G,1,1),p), p);
-      GEN b = modii(diviiexact(gcoeff(G,1,2),p), p);
-      GEN c = diviiexact(gcoeff(G,2,2),p);
-      GEN di= modii(subii(sqri(b), mulii(a,c)), p);
-      if (kronecker(di,p) >= 0)
+      /* Minimization was not possible so far. */
+      /* If n == 3 or 4, this proves the local nonsolubility at p. */
+      if (loc && (n == 3 || n == 4))
       {
-        if (DEBUGLEVEL >= 4) err_printf("    case 2.2\n");
-        sol = signe(a)? mkcol2(Fp_sub(Fp_sqrt(di,p), b, p), a): vec_ei(2,1);
+        if (DEBUGLEVEL >= 1) err_printf(" no local solution at %Ps\n",p);
+        return p;
       }
+      vectrunc_append(faP, p);
+      vecsmalltrunc_append(faE, vp);
+      break;
     }
-    if (sol)
-    {
-      long j;
-      sol = FpC_center(sol, p, shifti(p,-1));
-      sol = Q_primpart(sol);
-      if (DEBUGLEVEL >= 4) err_printf("    sol = %Ps\n", sol);
-      Ker = completebasis(vecextend(sol,n), 1);
-      for(j=1; j<n; j++) gel(Ker,j) = ZC_Z_mul(gel(Ker,j), p);
-      G = ZsymM_Z_divexact(qf_apply_ZM(G, Ker), sqri(p));
-      U = RgM_Rg_div(RgM_mul(U,Ker), p);
-      E[i] -= 2;
-      i--; continue; /* same p */
-    }
-    /* Now 1 <= vp = dimKer <= 2 and kernel contains no vector with norm p^2 */
-    /* exchanging kernel and image makes minimization easier ? */
-    m = (n-3)/2;
-    d = ZM_det(G); if (odd(m)) d = negi(d);
-    if ((vp==1 && kronecker(gmod(gdiv(negi(d), gcoeff(G,1,1)),p), p) >= 0)
-     || (vp==2 && odd(n) && n >= 5)
-     || (vp==2 && !odd(n) && kronecker(modii(diviiexact(d,sqri(p)), p),p) < 0))
-    {
-      long j;
-      if (DEBUGLEVEL >= 4) err_printf("    case 3\n");
-      Ker = matid(n);
-      for (j = dimKer+1; j <= n; j++) gcoeff(Ker,j,j) = p;
-      G = ZsymM_Z_divexact(qf_apply_ZM(G, Ker), p);
-      U = RgM_mul(U,Ker);
-      E[i] -= 2*dimKer-n;
-      i--; continue; /* same p */
-    }
-
-    /* Minimization was not possible so far. */
-    /* If n == 3 or 4, this proves the local nonsolubility at p. */
-    if (n == 3 || n == 4)
-    {
-      if (DEBUGLEVEL >= 1) err_printf(" no local solution at %Ps\n",p);
-      return(p);
-    }
-    vectrunc_append(faP, p);
-    vecsmalltrunc_append(faE, vp);
-    av = avma;
+    if (DEBUGLEVEL >= 3) err_printf("\n");
   }
   if (!U) U = matid(n);
   else
   { /* apply LLL to avoid coefficient explosion */
-    aux = lllint(Q_primpart(U));
-    G = qf_apply_ZM(G,aux);
-    U = RgM_mul(U,aux);
+    GEN u = lllint(Q_primpart(U));
+    G = qf_apply_ZM(G, u);
+    U = QM_mul(U, u);
   }
   return mkvec4(G, U, faP, faE);
 }
 
+/* assume G square integral */
+static void
+check_symmetric(GEN G)
+{
+  long i,j, l = lg(G);
+  for (i = 1; i < l; i++)
+    for(j = 1; j < i; j++)
+      if (!equalii(gcoeff(G,i,j), gcoeff(G,j,i)))
+        pari_err_TYPE("qfsolve [not symmetric]",G);
+}
+
+GEN
+qfminimize(GEN G)
+{
+  pari_sp av = avma;
+  GEN d, F, H;
+  long n = lg(G)-1;
+  if (typ(G) != t_MAT) pari_err_TYPE("qfminimize", G);
+  if (n == 0) pari_err_DOMAIN("qfminimize", "dimension" , "=", gen_0, G);
+  if (n != nbrows(G)) pari_err_DIM("qfminimize");
+  G = Q_primpart(G); RgM_check_ZM(G, "qfminimize");
+  check_symmetric(G);
+  d = ZM_det(G);
+  F = absZ_factor(d);
+  H = qfminimize_fact(G, gel(F,1),  ZV_to_zv(gel(F,2)), 0);
+  return gerepilecopy(av, mkvec2(gel(H,1), gel(H,2)));
+}
+
 /* CLASS GROUP COMPUTATIONS */
 
-/* Compute the square root of the quadratic form q of discriminant D. Not
- * fully implemented; it only works for detqfb squarefree except at 2, where
- * the valuation is 2 or 3.
- * mkmat2(P,zv_to_ZV(E)) = factor(2*abs(det q)) */
+/* Compute the square root of the quadratic form q of discriminant D = 4 * md
+ * Not fully implemented; only works for D squarefree except at 2, where the
+ * valuation is 2 or 3. Finally, [P,E] = factor(2*abs(D)) if valuation is 3 and
+ * factor(abs(D / 4)) otherwise */
 static GEN
-qfbsqrt(GEN D, GEN q, GEN P, GEN E)
+qfbsqrt(GEN D, GEN md, GEN q, GEN P)
 {
-  GEN a = gel(q,1), b = shifti(gel(q,2),-1), c = gel(q,3), mb = negi(b);
-  GEN m,n, aux,Q1,M, A,B,C;
-  GEN d = subii(mulii(a,c), sqri(b));
-  long i;
+  GEN a = gel(q,1), b = shifti(gel(q,2),-1), c = gel(q,3), B = negi(b);
+  GEN m, n, Q, M, N, d = negi(md); /* ac - b^2 */
+  long i, lP = lg(P);
 
-  /* 1st step: solve m^2 = a (d), m*n = -b (d), n^2 = c (d) */
-  m = n = mkintmod(gen_0,gen_1);
-  E[1] -= 3;
+  /* 1) solve m^2 = a, m*n = -b, n^2 = c in Z/dZ => q(n,m) = 0 mod d */
+  M = cgetg(lP, t_VEC);
+  N = cgetg(lP, t_VEC);
   for (i = 1; i < lg(P); i++)
   {
-    GEN p = gel(P,i), N, M;
-    if (!E[i]) continue;
-    if (dvdii(a,p)) {
-      aux = Fp_sqrt(c, p);
-      N = aux;
-      M = Fp_div(mb, aux, p);
-    } else {
-      aux = Fp_sqrt(a, p);
-      M = aux;
-      N = Fp_div(mb, aux, p);
-    }
-    n = chinese(n, mkintmod(N,p));
-    m = chinese(m, mkintmod(M,p));
+    GEN p = gel(P,i);
+    if (dvdii(a,p)) { n = Fp_sqrt(c, p); m = Fp_div(B, n, p); }
+    else            { m = Fp_sqrt(a, p); n = Fp_div(B, m, p); }
+    gel(M, i) = m;
+    gel(N, i) = n;
   }
-  m = centerlift(m);
-  n = centerlift(n);
-  if (DEBUGLEVEL >=4) err_printf("    [m,n] = [%Ps, %Ps]\n",m,n);
+  m = ZV_chinese_center(M, P, NULL);
+  n = ZV_chinese_center(N, P, NULL);
 
-  /* 2nd step: build Q1, with det=-1 such that Q1(x,y,0) = G(x,y) */
-  A = diviiexact(subii(sqri(n),c), d);
-  B = diviiexact(addii(b, mulii(m,n)), d);
-  C = diviiexact(subii(sqri(m), a), d);
-  Q1 = mkmat3(mkcol3(A,B,n), mkcol3(B,C,m), mkcol3(n,m,d));
-  Q1 = gneg(adj(Q1));
+  /* 2) build Q, with det=-1 such that Q(x,y,0) = G(x,y) */
+  N = diviiexact(addii(mulii(a,n), mulii(b,m)), d);
+  M = diviiexact(addii(mulii(b,n), mulii(c,m)), d);
+  Q = diviiexact(subiu(addii(mulii(m,M), mulii(n,N)), 1), d); /*(q(n,m)-d)/d^2 */
+  Q = mkmat3(mkcol3(a,b,N), mkcol3(b,c,M), mkcol3(N,M,Q)); /* det = -1 */
 
-  /* 3rd step: reduce Q1 to [0,0,-1;0,1,0;-1,0,0] */
-  M = qflllgram_indefgoon2(Q1);
+  /* 3) reduce Q to [0,0,-1; 0,1,0; -1,0,0] */
+  M = qflllgram_indefgoon2(Q);
   if (signe(gel(M,1)) < 0) M = ZC_neg(M);
   a = gel(M,1);
   b = gel(M,2);
   c = gel(M,3);
-  if (mpodd(a))
-    return qfb(D, a, shifti(b,1), shifti(c,1));
-  else
-    return qfb(D, c, shifti(negi(b),1), shifti(a,1));
+  if (!mpodd(a)) { swap(a, c); togglesign_safe(&b); }
+  return mkqfb(a, shifti(b,1), shifti(c,1), D);
 }
 
 /* \prod gen[i]^e[i] as a Qfb, e in {0,1}^n nonzero */
 static GEN
-qfb_factorback(GEN D, GEN gen, GEN e)
+qfb_factorback(GEN gen, GEN e, GEN isqrtD)
 {
   GEN q = NULL;
   long j, l = lg(gen), n = 0;
   for (j = 1; j < l; j++)
     if (e[j]) { n++; q = q? qfbcompraw(q, gel(gen,j)): gel(gen,j); }
-  return (n <= 1)? q: qfbreduce(D, q);
+  return (n <= 1)? q: qfbred0(q, 0, isqrtD, NULL);
 }
 
-/* unit form, assuming 4 | D */
+/* unit form discriminant 4d */
 static GEN
-id(GEN D)
-{ return mkmat2(mkcol2(gen_1,gen_0),mkcol2(gen_0,shifti(negi(D),-2))); }
+id(GEN d)
+{ return mkmat2(mkcol2(gen_1,gen_0),mkcol2(gen_0,negi(d))); }
 
 /* Shanks/Bosma-Stevenhagen algorithm to compute the 2-Sylow of the class
  * group of discriminant D. Only works for D = fundamental discriminant.
@@ -651,44 +656,44 @@ id(GEN D)
 static GEN
 quadclass2(GEN D, GEN P2D, GEN E2D, GEN Pm2D, GEN W, int n_is_4)
 {
-  GEN gen, Wgen, U2;
-  long i, n, r, m, vD;
+  GEN U2 = NULL, gen, Wgen, isqrtD, d;
+  long i, r, m;
+  int splice2 = mpodd(D);
 
-  if (mpodd(D))
-  {
-    D = shifti(D,2);
-    E2D = shallowcopy(E2D);
-    E2D[1] = 3;
+  if (!splice2) d = shifti(D,-2); else { d = D; D = shifti(D,2); }
+  /* D = 4d */
+  if (zv_equal0(W)) return id(d);
+  r = lg(Pm2D) - 4; /* >= 0 since W != 0 */
+  m = (signe(D) > 0)? r+1: r;
+  if (n_is_4)
+  { /* n = 4: look among forms of type q or 2*q, since Q can be imprimitive */
+    U2 = hilberts(gen_2, d, Pm2D);
+    if (zv_equal(U2,W)) return gmul2n(id(d),1);
   }
-  if (zv_equal0(W)) return id(D);
 
-  n = lg(Pm2D)-1; /* >= 3 since W != 0 */
-  r = n-3;
-  m = (signe(D)>0)? r+1: r;
-  /* n=4: look among forms of type q or 2*q, since Q can be imprimitive */
-  U2 = n_is_4? mkmat(hilberts(gen_2, D, Pm2D, lg(Pm2D))): NULL;
-  if (U2 && zv_equal(gel(U2,1),W)) return gmul2n(id(D),1);
-
-  gen = cgetg(m+1, t_VEC);
-  for (i = 1; i <= m; i++) /* no need to look at Pm2D[1]=-1, nor Pm2D[2]=2 */
-  {
-    GEN p = gel(Pm2D,i+2), d;
-    long vp = Z_pvalrem(D,p, &d);
-    gel(gen,i) = qfb(D, powiu(p,vp), gen_0, negi(shifti(d,-2)));
+  gen = cgetg(m+2, t_VEC);
+  for (i = 1; i <= m; i++)
+  { /* no need to look at P2D[1]=2*/
+    GEN q = powiu(gel(P2D,i+1), E2D[i+1]);
+    gel(gen,i) = mkqfb(q, gen_0, negi(diviiexact(d,q)), D);
   }
-  vD = Z_lval(D,2);  /* = 2 or 3 */
-  if (vD == 2 && smodis(D,16) != 4)
+  if (!mpodd(d))
   {
-    GEN q2 = qfb(D, gen_2,gen_2, shifti(subsi(4,D),-3));
-    m++; r++; gen = vec_append(gen, q2);
+    gel(gen, ++m) = mkqfb(gen_2, gen_0, negi(shifti(d,-1)), D);
+    r++;
   }
-  if (vD == 3)
+  else if (Mod4(d) != 1)
   {
-    GEN q2 = qfb(D, gen_2,gen_0, negi(shifti(D,-3)));
-    m++; r++; gen = vec_append(gen, q2);
+    gel(gen, ++m) = mkqfb(gen_2, gen_2, shifti(subsi(1,d),-1), D);
+    r++;
   }
-  if (!r) return id(D);
-  Wgen = qflocalinvariants(gen,Pm2D);
+  else setlg(gen, m+1);
+  if (!r) return id(d);
+  /* remove 2^3; leave alone 2^4 */
+  if (splice2) P2D = vecsplice(P2D, 1);
+  Wgen = cgetg(m+1, t_MAT);
+  for (i = 1; i <= m; i++) gel(Wgen,i) = hilberts(gmael(gen,i,1), d, Pm2D);
+  isqrtD = signe(D) > 0? sqrti(D) : NULL;
   for(;;)
   {
     GEN Wgen2, gen2, Ker, indexim = gel(Flm_indexrank(Wgen,2), 2);
@@ -697,13 +702,12 @@ quadclass2(GEN D, GEN P2D, GEN E2D, GEN Pm2D, GEN W, int n_is_4)
     {
       GEN W2 = Wgen, V;
       if (lg(indexim) < lg(Wgen)) W2 = vecpermute(Wgen,indexim);
-      if (U2) W2 = shallowconcat(W2,U2);
-      V = Flm_Flc_invimage(W2, W,2);
-      if (V) {
-        GEN Q = qfb_factorback(D, vecpermute(gen,indexim), V);
-        Q = gtomat(Q);
-        if (U2 && V[lg(V)-1]) Q = gmul2n(Q,1);
-        return Q;
+      if (U2) W2 = vec_append(W2,U2);
+      V = Flm_Flc_invimage(W2, W, 2);
+      if (V)
+      {
+        GEN Q = qfb_factorback(vecpermute(gen,indexim), V, isqrtD);
+        return (U2 && V[lg(V)-1])? qfbmat2(Q): qfbmat(Q);
       }
     }
     Ker = Flm_ker(Wgen,2); dKer = lg(Ker)-1;
@@ -711,10 +715,9 @@ quadclass2(GEN D, GEN P2D, GEN E2D, GEN Pm2D, GEN W, int n_is_4)
     Wgen2 = cgetg(m+1, t_MAT);
     for (i = 1; i <= dKer; i++)
     {
-      GEN q = qfb_factorback(D, gen, gel(Ker,i));
-      q = qfbsqrt(D,q,P2D,E2D);
-      gel(gen2,i) = q;
-      gel(Wgen2,i) = gel(qflocalinvariants(q,Pm2D), 1);
+      GEN q = qfb_factorback(gen, gel(Ker,i), isqrtD);
+      gel(gen2,i) = q = qfbsqrt(D, d, q, P2D);
+      gel(Wgen2,i) = hilberts(gel(q,1), d, Pm2D);
     }
     for (; i <=m; i++)
     {
@@ -816,25 +819,11 @@ qfsolvemodp(GEN G, GEN p)
   return ZC_add(x1, ZC_lincomb(r,s,x2,x3));
 }
 
-/* assume G square integral */
-static void
-check_symmetric(GEN G)
-{
-  long i,j, l = lg(G);
-  for (i = 1; i < l; i++)
-    for(j = 1; j < i; j++)
-      if (!equalii(gcoeff(G,i,j), gcoeff(G,j,i)))
-        pari_err_TYPE("qfsolve [not symmetric]",G);
-}
-
-/* Given a square matrix G of dimension n >= 1, */
-/* solves over Z the quadratic equation X^tGX = 0. */
-/* G is assumed to have integral coprime coefficients. */
-/* The solution might be a vectorv or a matrix. */
-/* If no solution exists, returns an integer, that can */
-/* be a prime p such that there is no local solution at p, */
-/* or -1 if there is no real solution, */
-/* or 0 in some rare cases. */
+/* Given a square rational matrix G of dimension n >= 1, solves over Z the
+ * quadratic equation X^tGX = 0. The solution is a t_VEC (a solution) or a
+ * t_MAT (totally isotropic subspace). If no solution exists, returns an
+ * integer: a prime p (no local solution at p), or -1 (no real solution), or
+ * -2 (n = 2 and -deg(G) not a square). */
 static  GEN
 qfsolve_i(GEN G)
 {
@@ -892,10 +881,10 @@ qfsolve_i(GEN G)
   /* P,E = factor(|det(G)|) */
 
   /* Minimization and local solubility */
-  Min = qfminimize(G, P, E);
+  Min = qfminimize_fact(G, P, E, 1);
   if (typ(Min) == t_INT) return Min;
 
-  M = RgM_mul(M, gel(Min,2));
+  M = QM_mul(M, gel(Min,2));
   G = gel(Min,1);
   P = gel(Min,3);
   E = gel(Min,4);
@@ -909,9 +898,9 @@ qfsolve_i(GEN G)
   /* Reduction of G and search for trivial solutions. */
   /* When |det G|=1, such trivial solutions always exist. */
   U = qflllgram_indef(G,0,&fail);
-  if(typ(U) == t_COL) return Q_primpart(RgM_RgC_mul(M,U));
+  if (typ(U) == t_COL) return Q_primpart(RgM_RgC_mul(M,U));
   G = gel(U,1);
-  M = RgM_mul(M, gel(U,2));
+  M = QM_mul(M, gel(U,2));
   /* P,E = factor(|det(G))| */
 
   /* If n >= 6 is even, need to increment the dimension by 1 to suppress all
@@ -928,11 +917,11 @@ qfsolve_i(GEN G)
     GEN aux;
     long i;
     codim = 1; n++;
-    /* largest square divisor of d */
     aux = gen_1;
     for (i = 1; i <= np; i++)
       if (E[i] == 2) { aux = mulii(aux, gel(P,i)); E[i] = 3; }
-    /* Choose sign(aux) so as to balance the signature of G1 */
+    /* aux = largest square divisor of d; choose sign(aux) so as to balance
+     * the signature of G1 */
     if (signG[1] > signG[2])
     {
       signG[2]++;
@@ -942,7 +931,7 @@ qfsolve_i(GEN G)
       signG[1]++;
     G1 = shallowmatconcat(diagonal_shallow(mkvec2(G,aux)));
     /* P,E = factor(|det G1|) */
-    Min = qfminimize(G1, P, E);
+    Min = qfminimize_fact(G1, P, E, 1);
     G1 = gel(Min,1);
     M1 = gel(Min,2);
     P = gel(Min,3);
@@ -951,36 +940,34 @@ qfsolve_i(GEN G)
   }
 
   /* now, d is squarefree */
-  if (!np)
-  { /* |d| = 1 */
-     G2 = G1;
-     M2 = NULL;
-  }
+  if (!np) { G2 = G1; M2 = NULL; } /* |d| = 1 */
   else
   { /* |d| > 1: increment dimension by 2 */
-    GEN factdP, factdE, W;
-    long i, lfactdP;
+    GEN factdP, factdE, W, v = NULL;
+    long i, lfactdP, lP;
     codim += 2;
     d = ZV_prod(P); /* d = abs(matdet(G1)); */
     if (odd(signG[2])) togglesign_safe(&d); /* d = matdet(G1); */
-    /* solubility at 2 (this is the only remaining bad prime). */
-    if (n == 4 && smodis(d,8) == 1 && qflocalinvariant(G,gen_2) == 1)
-      return gen_2;
-
+    if (n == 4)
+    { /* solubility at 2, the only remaining bad prime */
+      v = det_minors(G1);
+      if (Mod8(d) == 1 && witt(v, gen_2) == 1) return gen_2;
+    }
     P = shallowconcat(mpodd(d)? mkvec2(NULL,gen_2): mkvec(NULL), P);
     /* build a binary quadratic form with given Witt invariants */
-    W = const_vecsmall(lg(P)-1, 0);
+    lP = lg(P); W = const_vecsmall(lP-1, 0);
     /* choose signature of Q (real invariant and sign of the discriminant) */
     dQ = absi(d);
     if (signG[1] > signG[2]) togglesign_safe(&dQ); /* signQ = [2,0]; */
-    if (n == 4 && smodis(dQ,4) != 1) dQ = shifti(dQ,2);
+    if (n == 4 && Mod4(dQ) != 1) dQ = shifti(dQ,2);
     if (n >= 5) dQ = shifti(dQ,3);
 
     /* p-adic invariants */
     if (n == 4)
     {
-      GEN t = qflocalinvariants(ZM_neg(G1),P);
-      for (i = 3; i < lg(P); i++) W[i] = ucoeff(t,i,1);
+      togglesign(gel(v,2));
+      togglesign(gel(v,4)); /* v = det_minors(-G1) */
+      for (i = 3; i < lP; i++) W[i] = witt(v, gel(P,i)) < 0;
     }
     else
     {
@@ -988,11 +975,9 @@ qfsolve_i(GEN G)
       GEN t;
       if (odd((n-3)/2)) s = -s;
       t = s > 0? utoipos(8): utoineg(8);
-      for (i = 3; i < lg(P); i++)
-        W[i] = hilbertii(t, gel(P,i), gel(P,i)) > 0;
+      for (i = 3; i < lP; i++) W[i] = hilbertii(t, gel(P,i), gel(P,i)) > 0;
     }
-    /* for p = 2, the choice is fixed from the product formula */
-    W[2] = Flv_sum(W, 2);
+    W[2] = Flv_sum(W, 2); /* for p = 2, use product formula */
 
     /* Construction of the 2-class group of discriminant dQ until some product
      * of the generators gives the desired invariants. */
@@ -1008,7 +993,7 @@ qfsolve_i(GEN G)
     detG2 = mulii(d, ZM_det(Q));
     for (i = 1; i < lfactdP; i++) factdE[i] = Z_pval(detG2, gel(factdP,i));
     /* factdP,factdE = factor(|det G2|) */
-    Min = qfminimize(G2, factdP,factdE);
+    Min = qfminimize_fact(G2, factdP,factdE,1);
     M2 = gel(Min,2);
     G2 = gel(Min,1);
   }
@@ -1021,40 +1006,34 @@ qfsolve_i(GEN G)
     pari_err_IMPL("qfsolve, dim >= 10");
   solG2 = vecslice(gel(solG2,2), 1, dim-1);
 
-  if (!M2)
-    solG1 = solG2;
+  if (!M2) solG1 = solG2;
   else
   { /* solution of G1 is simultaneously in solG2 and x[n+1] = x[n+2] = 0*/
     GEN K;
-    solG1 = RgM_mul(M2,solG2);
-    K = ker(rowslice(solG1,n+1,n+2));
-    solG1 = RgM_mul(rowslice(solG1,1,n), K);
+    solG1 = QM_mul(M2,solG2);
+    K = QM_ker(rowslice(solG1,n+1,n+2));
+    solG1 = QM_mul(rowslice(solG1,1,n), K);
   }
-  if (!M1)
-    sol = solG1;
+  if (!M1) sol = solG1;
   else
   { /* solution of G1 is simultaneously in solG2 and x[n] = 0 */
     GEN K;
-    sol = RgM_mul(M1,solG1);
-    K = ker(rowslice(sol,n,n));
-    sol = RgM_mul(rowslice(sol,1,n-1), K);
+    sol = QM_mul(M1,solG1);
+    K = QM_ker(rowslice(sol,n,n));
+    sol = QM_mul(rowslice(sol,1,n-1), K);
   }
-  sol = Q_primpart(RgM_mul(M, sol));
+  sol = Q_primpart(QM_mul(M, sol));
   if (lg(sol) == 2) sol = gel(sol,1);
   return sol;
 }
 GEN
 qfsolve(GEN G)
-{
-  pari_sp av = avma;
-  return gerepilecopy(av, qfsolve_i(G));
-}
+{ pari_sp av = avma; return gerepilecopy(av, qfsolve_i(G)); }
 
 /* G is a symmetric 3x3 matrix, and sol a solution of sol~*G*sol=0.
  * Returns a parametrization of the solutions with the good invariants,
- * as a matrix 3x3, where each line contains
- * the coefficients of each of the 3 quadratic forms.
- * If fl!=0, the fl-th form is reduced. */
+ * as a 3x3 matrix, where each line contains the coefficients of each of the 3
+ * quadratic forms. If fl!=0, the fl-th form is reduced. */
 GEN
 qfparam(GEN G, GEN sol, long fl)
 {
