@@ -767,18 +767,28 @@ mpatanh(GEN x)
   shiftr_inplace(z, -1); return gerepileuptoleaf(av, z);
 }
 
+static long
+get_nmax(double u, double v, long prec)
+{
+  double d = 2 * log2(((double)v) / u); /* can be 0 due to rounding */
+  long nmax = -1;
+  if (d)
+  {
+    d = ceil(prec2nbits(prec) / d);
+    if (dblexpo(d) < BITS_IN_LONG) nmax = (long)d;
+  }
+  return nmax;
+}
 /* atanh(u/v) using binary splitting, 0 < u < v */
 GEN
 atanhuu(ulong u, ulong v, long prec)
 {
-  long i, nmax;
   GEN u2 = sqru(u), v2 = sqru(v);
-  double d = ((double)v) / u;
+  long i, nmax = get_nmax((double)u, (double)v, prec);
   struct abpq_res R;
   struct abpq A;
-  /* satisfies (2n+1) (v/u)^2n > 2^bitprec */
-  nmax = (long)ceil(prec2nbits(prec) / (2*log2(d)));
-  abpq_init(&A, nmax);
+  if (nmax < 0) pari_err_OVERFLOW("atanhuu");
+  abpq_init(&A, nmax); /* nmax satisfies (2n+1) (v/u)^2n > 2^bitprec */
   A.a[0] = A.b[0] = gen_1;
   A.p[0] = utoipos(u);
   A.q[0] = utoipos(v);
@@ -796,13 +806,11 @@ atanhuu(ulong u, ulong v, long prec)
 GEN
 atanhui(ulong u, GEN v, long prec)
 {
-  long i, nmax;
   GEN u2 = sqru(u), v2 = sqri(v);
-  double d = gtodouble(v) / u;
+  long i, nmax = get_nmax((double)u, gtodouble(v), prec);
   struct abpq_res R;
   struct abpq A;
-  /* satisfies (2n+1) (v/u)^2n > 2^bitprec */
-  nmax = (long)ceil(prec2nbits(prec) / (2*log2(d)));
+  if (nmax < 0) pari_err_OVERFLOW("atanhui");
   abpq_init(&A, nmax);
   A.a[0] = A.b[0] = gen_1;
   A.p[0] = utoipos(u);
@@ -847,13 +855,15 @@ gatanh(GEN x, long prec)
       return z;
     case t_FRAC:
     {
-      long ly, lz;
+      long ly, lz, e;
 
       y = gel(x,1); ly = lgefint(y);
       z = gel(x,2); lz = lgefint(z); if (ly > 3 && lz > 3) break;
       if (abscmpii(y, z) > 0) /* |y| > z; lz = 3 */
       {
         ulong u = z[2];
+        av = avma; e = expi((signe(y) < 0)? addii(y, z): subii(y, z));
+        set_avma(av); if (e < - prec2nbits(prec)) break;
         z = cgetg(3, t_COMPLEX); av = avma;
         a = ly == 3? atanhuu(u, y[2], prec): atanhui(u, y, prec);
         gel(z,1) = gerepileuptoleaf(av, a);
@@ -862,7 +872,8 @@ gatanh(GEN x, long prec)
       }
       else
       { /* |y| < z; ly = 3 */
-        av = avma;
+        av = avma; e = expi((signe(y) < 0)? addii(y, z): subii(y, z));
+        set_avma(av); if (e < - prec2nbits(prec)) break;
         a = lz == 3? atanhuu(y[2], z[2], prec): atanhui(y[2], z, prec);
         z = gerepileuptoleaf(av, a);
         if (signe(y) < 0) togglesign(z);
