@@ -716,6 +716,7 @@ zetamult_interpolate(GEN s, GEN t, long prec)
   long i, k, l, la, bit;
   GEN avec, v, V;
 
+  if (lg(s) == 1) return gen_1;
   if (!t) return zetamult(s, prec);
   avec = zetamultconvert_i(s, 1); k = zv_sum(avec);
   bit = prec2nbits(prec);
@@ -735,12 +736,12 @@ zetamult_interpolate(GEN s, GEN t, long prec)
   return gerepileupto(av, poleval(vecreverse(V),t));
 }
 
-
 GEN
-polylogmult(GEN a, GEN z, long prec)
+polylogmult_interpolate(GEN a, GEN z, GEN t, long prec)
 {
   pari_sp av = avma;
-  if (!z) return zetamult(a, prec);
+  GEN V, avec, A, AZ, Z;
+  long i, la, l;
   switch(typ(a))
   {
     case t_VEC:
@@ -749,6 +750,7 @@ polylogmult(GEN a, GEN z, long prec)
     default: pari_err_TYPE("polylogmult", a);
              return NULL;/*LCOV_EXCL_LINE*/
   }
+  if (!z) return zetamult_interpolate(a, t, prec);
   switch (typ(z))
   {
     case t_INT: case t_FRAC: case t_REAL: case t_COMPLEX:
@@ -759,32 +761,27 @@ polylogmult(GEN a, GEN z, long prec)
   }
   if (lg(z) != lg(a))
     pari_err_TYPE("polylogmult [#s != #z]", mkvec2(a,z));
-  return gerepilecopy(av, zetamultevec(aztoe(a,z,prec), prec));
-}
-
-GEN
-polylogmult_interpolate(GEN s, GEN zvec, GEN t, long prec)
-{
-  pari_sp av = avma;
-  GEN V, avec, A, AZ, Z;
-  long i, la, l;
-
-  if (!t) return polylogmult(s, zvec, prec);
-  if (!zvec) return zetamult_interpolate(s, t, prec);
-  avec = zetamultconvert_i(s, 1); la = lg(avec);
-  AZ = allstar2(avec, zvec);
+  if (!t) return gerepilecopy(av, zetamultevec(aztoe(a,z,prec), prec));
+  avec = zetamultconvert_i(a, 1); la = lg(avec);
+  AZ = allstar2(avec, z);
   A = gel(AZ, 1); l = lg(A);
   Z = gel(AZ, 2); V = zerovec(la-1);
   for (i = 1; i < l; i++)
   {
     pari_sp av2 = avma;
-    GEN a = gel(A,i), e = aztoe(a, gel(Z,i), prec);
-    long n = lg(a)-1; /* > 0 */
+    GEN ai = gel(A,i), e = aztoe(ai, gel(Z,i), prec);
+    long n = lg(ai)-1; /* > 0 */
     gel(V,n) = gerepileupto(av2, gadd(gel(V,n), zetamultevec(e, prec)));
   }
   return gerepileupto(av, poleval(vecreverse(V),t));
 }
 
+GEN
+polylogmult(GEN a, GEN z, long prec)
+{
+  return polylogmult_interpolate(a, z, NULL, prec);
+}
+
 /**************************************************************/
 /*                           ALL MZV's                        */
 /**************************************************************/
@@ -984,6 +981,7 @@ zetamultall(long k, long flag, long prec)
 
   if (flag < 0 || flag > 15) pari_err_FLAG("zetamultall");
   if (k < 1) pari_err_DOMAIN("zetamultall", "k", "<", gen_1, stoi(k));
+  if (k == 1) return cgetg(1, t_VEC);
   if (k >= 64) pari_err_OVERFLOW("zetamultall");
   if (!(flag & 1L))
   { /* not star */
