@@ -5,7 +5,8 @@ This file is part of the PARI/GP package.
 
 PARI/GP is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
-Foundation. It is distributed in the hope that it will be useful, but WITHOUT
+Foundation; either version 2 of the License, or (at your option) any later
+version. It is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
@@ -138,7 +139,7 @@ addir_sign(GEN x, long sx, GEN y, long sy)
   z = (GEN)avma;
   y = addrr_sign(itor(x,l), sx, y, sy);
   ly = lg(y); while (ly--) *--z = y[ly];
-  avma = (pari_sp)z; return z;
+  set_avma((pari_sp)z); return z;
 }
 
 static GEN
@@ -167,7 +168,7 @@ addsr_sign(long x, GEN y, long sy)
   z = (GEN)avma;
   y = addrr_sign(stor(x,l), sx, y, sy);
   ly = lg(y); while (ly--) *--z = y[ly];
-  avma = (pari_sp)z; return z;
+  set_avma((pari_sp)z); return z;
 }
 
 GEN
@@ -229,7 +230,7 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
       GEN p1 = x; x = new_chunk(lx + lz + 1);
       shift_right(x,p1,2,lx, 0,m);
       if (extend) uel(x,lx) = uel(p1,lx-1) << sh;
-      avma = av; /* HACK: cgetr(lz) will not overwrite x */
+      set_avma(av); /* HACK: cgetr(lz) will not overwrite x */
     }
   }
   else
@@ -282,7 +283,7 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
     if (i==lx) return real_0_bit(ey+1 - prec2nbits(lx));
     f2 = (uel(y,i) > uel(x,i));
   }
-  /* result is non-zero. f2 = (y > x) */
+  /* result is nonzero. f2 = (y > x) */
   i = lz-1; z = cgetr(lz);
   if (f2)
   {
@@ -335,5 +336,5 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
   else if (j) shift_left(z,z,2,lz-1, 0,j);
   z[1] = evalsigne(sx) | evalexpo(ez);
   z[0] = evaltyp(t_REAL) | evallg(lz);
-  avma = (pari_sp)z; return z;
+  set_avma((pari_sp)z); return z;
 }
