@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.3.2.  */
+/* A Bison parser, made by GNU Bison 3.7.5.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -31,8 +31,9 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-/* Undocumented macros, especially those whose name start with YY_,
-   are private implementation details.  Do not rely on them.  */
+/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
+   especially those whose name start with YY_ or yy_.  They are
+   private implementation details that can be changed or removed.  */
 
 #ifndef YY_PARI_SRC_LANGUAGE_PARSE_H_INCLUDED
 # define YY_PARI_SRC_LANGUAGE_PARSE_H_INCLUDED
@@ -52,46 +53,51 @@
 extern int pari_debug;
 #endif
 
-/* Token type.  */
+/* Token kinds.  */
 #ifndef PARI_TOKENTYPE
 # define PARI_TOKENTYPE
   enum pari_tokentype
   {
-    KPARROW = 258,
-    KARROW = 259,
-    KDOTDOT = 260,
-    KPE = 261,
-    KSE = 262,
-    KME = 263,
-    KDE = 264,
-    KDRE = 265,
-    KEUCE = 266,
-    KMODE = 267,
-    KAND = 268,
-    KOR = 269,
-    KID = 270,
-    KEQ = 271,
-    KNE = 272,
-    KGE = 273,
-    KLE = 274,
-    KSRE = 275,
-    KSLE = 276,
-    KSR = 277,
-    KSL = 278,
-    KDR = 279,
-    KPP = 280,
-    KSS = 281,
-    KINTEGER = 282,
-    KREAL = 283,
-    KENTRY = 284,
-    KSTRING = 285,
-    SEQ = 286,
-    DEFFUNC = 287,
-    INT = 288,
-    LVAL = 289,
-    SIGN = 290,
-    DERIV = 291
+    PARI_EMPTY = -2,
+    PARI_EOF = 0,                  /* "end of file"  */
+    PARI_error = 256,              /* error  */
+    PARI_UNDEF = 257,              /* "invalid token"  */
+    KPARROW = 258,                 /* ")->"  */
+    KARROW = 259,                  /* "->"  */
+    KDOTDOT = 260,                 /* ".."  */
+    KPE = 261,                     /* "+="  */
+    KSE = 262,                     /* "-="  */
+    KME = 263,                     /* "*="  */
+    KDE = 264,                     /* "/="  */
+    KDRE = 265,                    /* "\\/="  */
+    KEUCE = 266,                   /* "\\="  */
+    KMODE = 267,                   /* "%="  */
+    KAND = 268,                    /* "&&"  */
+    KOR = 269,                     /* "||"  */
+    KID = 270,                     /* "==="  */
+    KEQ = 271,                     /* "=="  */
+    KNE = 272,                     /* "!="  */
+    KGE = 273,                     /* ">="  */
+    KLE = 274,                     /* "<="  */
+    KSRE = 275,                    /* ">>="  */
+    KSLE = 276,                    /* "<<="  */
+    KSR = 277,                     /* ">>"  */
+    KSL = 278,                     /* "<<"  */
+    KDR = 279,                     /* "\\/"  */
+    KPP = 280,                     /* "++"  */
+    KSS = 281,                     /* "--"  */
+    KINTEGER = 282,                /* "integer"  */
+    KREAL = 283,                   /* "real number"  */
+    KENTRY = 284,                  /* "variable name"  */
+    KSTRING = 285,                 /* "character string"  */
+    SEQ = 286,                     /* SEQ  */
+    DEFFUNC = 287,                 /* DEFFUNC  */
+    INT = 288,                     /* INT  */
+    LVAL = 289,                    /* LVAL  */
+    SIGN = 290,                    /* SIGN  */
+    DERIV = 291                    /* DERIV  */
   };
+  typedef enum pari_tokentype pari_token_kind_t;
 #endif
 
 /* Value type.  */
