@@ -276,6 +276,7 @@ sieve_init1(long p, ratpoints_sieve_entry *se1, long b1, ratpoints_args *args1)
 static ratpoints_bit_array *
 sieve_init2(long p, ratpoints_sieve_entry *se1, long b1, ratpoints_args *args1)
 {
+  pari_sp av = avma;
   ratpoints_sieve_entry *se = se1;
   ratpoints_args *args = args1;
   int *isfs = se->is_f_square;
@@ -285,7 +286,7 @@ sieve_init2(long p, ratpoints_sieve_entry *se1, long b1, ratpoints_args *args1)
   long wp = p >> TWOPOTBITS_IN_LONG;
   long diff_shift = p & LONG_MASK;
   long diff = BITS_IN_LONG - diff_shift;
-  ulong help[(p>>TWOPOTBITS_IN_LONG) + 2];
+  ulong *help = (ulong *) new_chunk((p>>TWOPOTBITS_IN_LONG) + 2);
 
   /* initialize help */
   {
@@ -334,6 +335,7 @@ sieve_init2(long p, ratpoints_sieve_entry *se1, long b1, ratpoints_args *args1)
      CODE_INIT_SIEVE_COPY
     /* set sieve array */
     se->sieve[b] = (ratpoints_bit_array *)si;
+    set_avma(av);
     return (ratpoints_bit_array *)si;
   }
 }
@@ -1066,6 +1068,7 @@ sieving_info(ratpoints_args *args,
   long pn, pnp = 0;
   long n;
   long nbprime = lg(args->listprime)-1;
+  long * coeffs_mod_p;
   entry *prec = (entry*) stack_malloc(nbprime*sizeof(entry));
     /* This array is used for sorting in order to
        determine the `best' sieving primes. */
@@ -1075,11 +1078,10 @@ sieving_info(ratpoints_args *args,
   ulong bound = (1UL)<<(BITS_IN_LONG - args->bit_primes);
   pari_sp av = avma;
   squares = gen_squares(args->listprime);
-
+  coeffs_mod_p = (long *) new_chunk(degree+1);
   /* initialize sieve in se_buffer */
   for (pn = 1; pn <= args->num_primes; pn++)
   {
-    long coeffs_mod_p[degree+1]; /* The coefficients of f reduced modulo p */
     ulong a, p = args->listprime[pn], np;
     long n;
     int *is_f_square = args->int_next;
@@ -1234,9 +1236,9 @@ sift(long b, ratpoints_bit_array *survivors, ratpoints_args *args,
      int process(long, long, GEN, void*, int*), void *info)
 {
   pari_sp av = avma;
-  sieve_spec ssp[args->sp2];
   int do_setup = 1;
   long k, height = args->height, nb;
+  sieve_spec *ssp = (sieve_spec *) stack_malloc(args->sp2*sizeof(sieve_spec));
 
   if (odd(b) == 0) which_bits = num_odd; /* even denominator */
 
@@ -1409,9 +1411,9 @@ find_points_work(ratpoints_args *args,
    *         trailing coefficient is zero, |coeff. of x| = 1 */
   if (!(args->flags & RATPOINTS_NO_REVERSE))
   {
-    if (!odd(degree))
+    if (!odd(degree) && degree>0)
     {
-      if (signe(pel(c,0)) == 0)
+      if (signe(pel(c,0)) == 0 && signe(pel(c,1))!=0)
       { /* case 1 */
         long n;
         args->flags |= RATPOINTS_REVERSED;
@@ -1502,13 +1504,13 @@ find_points_work(ratpoints_args *args,
   {
     ratpoints_bit_array *survivors = (ratpoints_bit_array *)
       stack_malloc_align((args->array_size)*RBA_SIZE, RBA_SIZE);
+    long *bp_list = (long *) new_chunk(args->sp2);
     if (args->flags & (RATPOINTS_USE_SQUARES | RATPOINTS_USE_SQUARES1))
     {
       if (args->flags & RATPOINTS_USE_SQUARES)
       /* need only take squares as denoms */
       {
         long b, bb;
-        long bp_list[args->sp2];
         long last_b = args->b_low;
         long n;
         for (n = 0; n < args->sp2; n++)
@@ -1541,7 +1543,6 @@ find_points_work(ratpoints_args *args,
         long ld = lg(divisors);
         long k;
         long b, bb;
-        long bp_list[args->sp2];
 
         for (k = 1; k < ld; k++)
         {
@@ -1594,7 +1595,6 @@ find_points_work(ratpoints_args *args,
       {
         long *forb;
         long b;
-        long bp_list[args->sp2];
         long last_b = args->b_low;
         ulong b_bits;
         long n;
@@ -1664,7 +1664,6 @@ find_points_work(ratpoints_args *args,
       else
       {
         long b, n;
-        long bp_list[args->sp2];
         long last_b = args->b_low;
         for (n = 0; n < args->sp2; n++)
           bp_list[n] = mod(args->b_low, sieve_list[n]->p);
